--- Start of the Rule Requirement ---
{{rule_requirement}}
--- End of the Rule Requirement ---

in addition to the `Rule Requirement` provided above, you should notice that rule should always base on concrete truth, and always write rule with additional info, including the purpose, and the context.

The `Rule` instance you are going to generate must use the same language as the `Rule Requirement`, which is "{{getlang rule_requirement}}"!