<project_path>{{ absolute_code_path }}</project_path>

{{#if source_tree}}
    <source_tree>
        ```
        {{{source_tree}}}
        ```
    </source_tree>
{{/if}}

<files>
    {{#each files}}
        {{#if code}}
            <file>
                <path>{{path}}</path>
                <code>
                    {{{code}}}
                </code>
            </file>
        {{/if}}
    {{/each}}
</files>

{{#if instructions}}
    <instructions>
        {{{instructions}}}
    </instructions>
{{/if}}

<final_instruction>
    Consider the project path in
    <project_path>, {{#if source_tree}}the source tree in
    <source_tree>,{{/if}} and the files in
        <files>. Then, follow the instructions given in
            <instructions>. Take a deep breath and think step by step about how to best complete this task.
</final_instruction>