{{#if docs}}{{#each docs}}
----- Start of {{ @index }}th retrieved document -----
{{this}}
----- End of {{ @index }}th retrieved document -----
{{/each}}Content above is retrieved from the database, use them as reference if needed.{{/if}}