--- Start of RuleSet Generate requirement ---
{{ruleset_requirement}}
--- End of RuleSet Generate requirement ---

# Propose
Propose detailed and concise rule generation requirements that will be used to generate rules separately.

# Origin
Explain the origin and context of these rule requirements.

# Context
Provide additional context and constraints for the rule generation.

# Note
Based on ruleset requirement provided above, you need to break it down to several rule generation requirements, which will be used to generate rule separately.
Most importantly, `RuleSet Generate requirement` is absolute, you must make sure your proposed rules satisfy all of it.
The rule generation requirements each should be detailed and concise as much as possible and should be able to generate a valid rule.
Detail ignore is unacceptable!
the requirement for rule generation should use the same language as `RuleSet Generate requirement` does, which is {{getlang ruleset_requirement}}!