{{#if reference}}--- Start of reference content ---
{{reference}}
--- End of reference content ---
{{/if}}
# Fix Troubled Object

## Problem Description
{{problem}}

## Solution
{{solution}}

## Instructions
Please review the object described above and apply the provided solution to fix the issue. Ensure that the changes are accurate and align with the reference material provided.