{{#each this}}
### {{@key}}
- **Path:** {{path}}
- **Exists:** {{exists}}
- **Description:** {{description}}
- **Size:** {{size}}
- **Checksum:** {{checksum}}
{{#if content}}- **Lines:** {{lines}}
- **Content:**


```{{language}}
{{content}}
```{{/if}}


{{/each}}