{{block source "Extract Source"}}
{{#if extract_requirement}}
    {{block extract_requirement "Extract Requirement"}}{{/if}}
You need to extract `Extract Source` to an object.
{{#if extract_requirement}}`Extract Requirement` is highest and absolute executing rule, no violation is allowed!{{/if}}
{{#if align_language}}you MUST use `{{getlang source}}` as the written language.{{/if}}