{{original_q}}

--- Start of Response That Violated Instructions ---
{{original_a}}
--- End of Response That Violated Instructions ---
`Response That Violated Instructions` is a wrong Response, you need make a new one that can follow the requirement and constrains.

